/***************************************************************************
 *   Copyright (C) 2005 by Niklas Knutsson                                 *
 *   nq@altern.org                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QALCULATEPLOTDIALOG_H
#define QALCULATEPLOTDIALOG_H

#include <kdialog.h>
#include <qmap.h>
#include <qstring.h>
//Added by qt3to4:
#include <QLabel>

class KLineEdit;
class QCheckBox;
class QRadioButton;
class QSpinBox;
class Q3ButtonGroup;
class KComboBox;
class QPushButton;
class QTabWidget;
class K3ListView;
class QLabel;
class Q3ListViewItem;
class QalculateExpressionEdit;

#include "qalculate_kde_utils.h"

/**
@author Niklas Knutsson
*/
class QalculatePlotDialog : public KDialog {

	Q_OBJECT

public:

	QalculatePlotDialog(QWidget *parent = 0, const char *name = 0);
	virtual ~QalculatePlotDialog();

	QLabel *xVariableLabel;
	KLineEdit *seriesTitleEdit, *xVariableEdit, *minEdit, *maxEdit, *stepSizeEdit, *titleEdit, *xLabelEdit, *yLabelEdit;
	QalculateExpressionEdit *expressionEdit;
	QCheckBox *rowsButton, *gridButton, *fullBorderButton, *logXButton, *logYButton;
	QRadioButton *functionButton, *vectorMatrixButton, *pairedMatrixButton, *primaryButton, *secondaryButton, *samplingRateButton, *stepSizeButton, *colorButton, *monochromeButton;
	QSpinBox *samplingRateBox, *logXBox, *logYBox;
	Q3ButtonGroup *typeGroup, *rangeGroup;
	KComboBox *styleCombo, *smoothingCombo, *legendPlacementCombo;
	QPushButton *addButton, *modifyButton, *removeButton, *applyRangeButton, *applyAppearanceButton;
	QTabWidget *tabs;
	K3ListView *seriesView;

	bool generatePlot(PlotParameters &pp, std::vector<MathStructure> &y_vectors, std::vector<MathStructure> &x_vectors, std::vector<PlotDataParameters*> &pdps);
	void generatePlotSeries(MathStructure **x_vector, MathStructure **y_vector, int type, std::string str, std::string str_x);

	QMap<Q3ListViewItem*, int> itemStyle;
	QMap<Q3ListViewItem*, int> itemSmoothing;
	QMap<Q3ListViewItem*, int> itemType;
	QMap<Q3ListViewItem*, int> itemAxis;
	QMap<Q3ListViewItem*, bool> itemRows;
	QMap<Q3ListViewItem*, MathStructure*> itemXVector;
	QMap<Q3ListViewItem*, MathStructure*> itemYVector;
	QMap<Q3ListViewItem*, QString> itemXVariable;
	
protected slots:
	
	void slotHelp();	

public slots:

	void onClosing();
	void seriesItemRenamed(Q3ListViewItem*, const QString&, int);
	void seriesSelected();
	void rangeTypeChanged(int);
	void typeChanged(int);
	void savePlot();
	void expressionActivated();
	void addSeries();
	void modifySeries();
	void removeSeries();
	void applyRange();
	void applyAppearance();
	void updatePlot();
	void saveMode();

};

#endif
